from textual.app import App
from textual.views import GridView
from textual.widgets import Footer, Header
from textual import events

from textual.app import App

from pixelart_tui.canvas import Canvas, MouseStatus, Tools, Grid

from textual_inputs import TextInput
from pixelart_tui.file_tools import save_pxl, load_pxl, save_pal, load_pal

from pixelart_tui.my_messages import DebugStatus, ColorStatus, LoadSaveStatus
from pixelart_tui.my_widgets import (
    LoadSave,
    StatusWidget,
    ColorWidget,
    ColorBox,
    ToolsWidget,
)


class Layout(GridView):
    def __init__(
        self,
        w: int,
        h: int,
        canvas: Canvas,
        status: StatusWidget,
        c_red: ColorWidget,
        c_green: ColorWidget,
        c_blue: ColorWidget,
        c_box: ColorBox,
        name_input: TextInput,
        loadsave: LoadSave,
        *args,
        **kwargs,
    ) -> None:
        super().__init__(*args, **kwargs)
        self.w = w
        self.h = h
        self.canvas = canvas
        self.status = status
        self.c_red = c_red
        self.c_green = c_green
        self.c_blue = c_blue
        self.c_box = c_box
        self.name_input = name_input
        self.loadsave = loadsave

    def on_mount(self) -> None:
        # Attributes to store the current calculation
        self.grid.set_gap(2, 1)
        self.grid.set_gutter(1)
        self.grid.set_align("center", "center")

        # One pixel in y is twice size
        w, h = self.w, self.h // 2
        h1 = 18
        h2 = 3
        h3 = h - h1 - h2
        # Create rows / columns / areas
        self.grid.add_column("gamw", min_size=w, max_size=w)
        self.grid.add_row("gamh", min_size=h1, max_size=h1)
        self.grid.add_row("text", min_size=h2, max_size=h2)
        self.grid.add_row("colo", min_size=h3, max_size=h3)
        self.grid.add_column("panel", min_size=6, max_size=6, repeat=5)

        self.grid.add_areas(
            game="gamw,gamh-start|colo-end", panel="panel1-start|panel5-end,colo"
        )
        self.grid.add_areas(red="panel1,gamh", green="panel2,gamh", blue="panel3,gamh")
        self.grid.add_areas(text="panel1-start|panel3-end,text")
        self.grid.add_areas(box="panel4-start|panel5-end,gamh")
        self.grid.add_areas(loadsave="panel4-start|panel5-end,text")
        # Place out widgets in to the layout
        self.grid.place(game=self.canvas)
        self.grid.place(panel=ToolsWidget())
        self.grid.place(red=self.c_red)
        self.grid.place(green=self.c_green)
        self.grid.place(blue=self.c_blue)
        self.grid.place(text=self.name_input)
        self.grid.place(box=self.c_box)
        self.grid.place(loadsave=self.loadsave)


class SimpleApp(App):
    async def on_load(self, _: events.Load) -> None:
        await self.bind("q", "quit", "Quit")
        await self.bind("r", "reset()", "Reset")

    async def on_mount(self) -> None:
        w, h = 64, 64
        self.status = StatusWidget()
        self.canvas = Canvas(w, h, grid=Grid.g32x32)

        self.c_red = ColorWidget("red")
        self.c_green = ColorWidget("green")
        self.c_blue = ColorWidget("blue")
        self.c_box = ColorBox(0, 0, 0)

        self.name_input = TextInput(placeholder="file_name")
        self.loadsave = LoadSave()

        self.layout = Layout(
            w,
            h,
            self.canvas,
            self.status,
            self.c_red,
            self.c_green,
            self.c_blue,
            self.c_box,
            self.name_input,
            self.loadsave,
        )

        style_fh = "white on rgb(111,22,44)"
        await self.view.dock(Header(style=style_fh), edge="top")
        # Fix style of footer
        await self.view.dock(Footer(), edge="bottom")
        await self.view.dock(self.layout, edge="left")

    async def handle_mouse_status(self, message: MouseStatus):
        self.status.pos = message.pos

    async def handle_debug_status(self, message: DebugStatus):
        self.status.debug = message.mes

    async def handle_color_status(self, message: ColorStatus):
        self.c_box.update(**message.color)
        r, g, b = self.c_box.get_color().get_truecolor()
        self.c_red.update(r)
        self.c_green.update(g)
        self.c_blue.update(b)
        self.canvas.set_color(self.c_box.get_color())

    async def handle_load_save_status(self, message: LoadSaveStatus):
        format = {"pxl", "pal"}
        file_name = self.name_input.value
        end = file_name.split(".")[-1]
        if not end in format:
            return

        if end == "pxl":
            if message.loadsave == "Save":
                save_pxl(self.canvas.matrix, file_name)
            elif message.loadsave == "Load":
                load_pxl(self.canvas.matrix, file_name)
                self.canvas.refresh()

        elif end == "pal":
            if message.loadsave == "Save":
                save_pal(self.c_box.saved, file_name)
            elif message.loadsave == "Load":
                load_pal(self.c_box.saved, file_name)
                self.canvas.set_color(self.c_box.get_color())
                self.c_box.refresh()

    async def handle_my_button_pressed(self, message):
        if message.value == "8":
            self.canvas.grid = Grid.g8x8
            self.canvas.refresh()
        if message.value == "16":
            self.canvas.grid = Grid.g16x16
            self.canvas.refresh()
        if message.value == "32":
            self.canvas.grid = Grid.g32x32
            self.canvas.refresh()
        if message.value == "new":
            self.canvas.set_matrix()
            self.canvas.refresh()
        if message.value == "pick":
            self.canvas.set_tool(Tools.PICK)
        if message.value == "paint":
            self.canvas.set_tool(Tools.PAINT)  # color = self.canvas.get_color()
            # self.canvas.set_color()

    async def handle_color_under_mouse(self, message):
        if not message.change:
            self.c_box.display(message.color)
        else:
            r, g, b = message.color.get_truecolor()
            self.c_box.update(r, g, b)
            self.c_red.update(r)
            self.c_green.update(g)
            self.c_blue.update(b)


if __name__ == "__main__":
    SimpleApp.run()



from __future__ import annotations

from rich import box
from rich.align import Align
from rich.console import Console, ConsoleOptions, RenderResult, RenderableType
from rich.style import StyleType

from textual import events
from textual.message import Message
from textual.reactive import Reactive
from textual.widget import Widget

from textual.widgets import ButtonPressed, Button
from rich.panel import Panel


class MyButtonPressed(Message):
    def __init__(self, sender: Widget, value: str):
        super().__init__(sender)
        self.value = value


class MyButton(Button):
    mouse_over: Reactive[bool] = Reactive(False)
    has_focus: Reactive[bool] = Reactive(False)

    def __init__(
        self,
        label: RenderableType,
        name: str | None = None,
        style: StyleType = "white on dark_blue",
        value: str = "",
    ):
        super().__init__(label, name, style)
        self.value = value

    def render(self) -> RenderableType:
        return self

    def __rich_console__(
        self, console: Console, options: ConsoleOptions
    ) -> RenderResult:
        yield Panel(
            Align.center(
                self.label,
            ),
            border_style="green" if self.mouse_over else "blue",
            box=box.HEAVY if self.has_focus else box.ROUNDED,
            style=self.button_style,
            # height=height,
        )

    async def on_click(self, event: events.Click) -> None:
        await self.emit(MyButtonPressed(self, self.value))

    async def on_enter(self, event: events.Enter) -> None:
        self.mouse_over = True

    async def on_leave(self, event: events.Leave) -> None:
        self.mouse_over = False
        self.has_focus = False

    async def on_mouse_down(self, _) -> None:
        self.has_focus = True

    async def on_mouse_up(self, _) -> None:
        self.has_focus = False



from rich.align import Align
from rich.console import (
    RenderableType,
)
from rich.align import Align
from rich.console import RenderableType
from textual.reactive import Reactive
from textual.views import GridView
from textual.widget import Widget
from textual import events

from rich.panel import Panel

from rich.segment import Segment
from rich.style import Style

from rich.color import Color
from rich.console import Console
from rich.console import ConsoleOptions
from rich.console import (
    Console,
    ConsoleOptions,
    RenderResult,
    RenderableType,
)
from pixelart_tui.my_button import MyButton
from typing import List

from pixelart_tui.my_messages import DebugStatus, ColorStatus, LoadSaveStatus


def minmax(a, mn, mx):
    return max(min(mx, a), mn)


class LoadSave(Widget):
    chosen: Reactive = Reactive(0)

    def __rich_console__(
        self, console: Console, options: ConsoleOptions
    ) -> RenderResult:
        ...
        u = "────"
        ub = "━━━━"

        u1 = ub if self.chosen == 1 else u
        u2 = ub if self.chosen == 2 else u
        t1 = "[b]Load[/b]" if self.chosen == 1 else "Load"
        t2 = "[b]Save[/b]" if self.chosen == 2 else "Save"
        yield (f"\n" f" {t1}    {t2} \n" f" {u1}    {u2}")

    def render(self) -> RenderableType:
        return self

    async def on_click(self, event: events.Click) -> None:
        if self.chosen == 1:
            await self.emit(LoadSaveStatus(self, "Load"))
        elif self.chosen == 2:
            await self.emit(LoadSaveStatus(self, "Save"))

    async def on_mouse_move(self, event: events.MouseMove) -> None:
        if 1 <= event.x <= 4:
            self.chosen = 1
        elif 8 <= event.x <= 12:
            self.chosen = 2
        else:
            self.chosen = 0

    async def on_leave(self, _):
        self.chosen = 0


class ColorBox(Widget):
    red: Reactive[int] = Reactive(0)
    green: Reactive[int] = Reactive(0)
    blue: Reactive[int] = Reactive(0)
    saved: Reactive[List[Color]] = Reactive(
        [Color.from_rgb(0, 0, 0) for _ in range(35)]
    )
    chosen: Reactive = Reactive(0)
    to_display: Reactive = Reactive(Color.from_rgb(0, 0, 0))

    def __init__(self, r: int, g: int, b: int, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)
        self.red = r
        self.green = g
        self.blue = b
        self.saved[self.chosen]

    def __rich_console__(
        self, console: Console, options: ConsoleOptions
    ) -> RenderResult:
        max_h = self._size.height
        max_w = self._size.width
        col = ["red", "green", "blue"]
        val = self.get_color().get_truecolor()
        for y in range(0, 3):
            for x in range(3):
                yield Segment("██", Style(color=self.to_display))
            yield Segment("|")
            yield Segment(f"{col[y][0]}: {val[y]}", Style(color=col[y]))

            yield Segment.line()
        yield Segment("─" * max_w)
        yield Segment.line()
        for y in range(0, 7):
            for x in range(5):
                c = self.saved[y * 5 + x]
                if y * 5 + x == self.chosen:
                    yield Segment("━━", Style(color=c))
                else:
                    yield Segment("██", Style(color=c))
                yield Segment("│")
                # yield Segment(" ")
            yield Segment.line()
            # yield Segment("──┼"*(max_w//3))
            yield Segment("── " * (max_w // 3 + 1))
            if y < 5:
                yield Segment("  +" * (max_w // 3 + 1))
            else:
                yield Segment("---" * (max_w // 3 + 1))
            yield Segment.line()

    def render(self) -> RenderableType:
        return self

    def update(self, red=None, green=None, blue=None):
        if red is not None:
            self.red = red
        if green is not None:
            self.green = green
        if blue is not None:
            self.blue = blue
        self.saved[self.chosen] = Color.from_rgb(self.red, self.green, self.blue)
        self.to_display = self.saved[self.chosen]

    def display(self, color):
        self.to_display = color
        self.refresh()

    def get_color(self):
        r, g, b = self.saved[self.chosen].get_truecolor()
        return Color.from_rgb(r, g, b)

    async def on_click(self, event: events.Click) -> None:
        # 4, 0
        y, x = event.y, event.x
        if y >= 4 and y % 2 == 0:
            if x % 3 != 2:
                self.chosen = (y - 4) // 2 * 5 + x // 3
                self.red, self.green, self.blue = self.saved[
                    self.chosen
                ].get_truecolor()
        await self.emit(
            ColorStatus(
                self,
                {
                    "red": self.red,
                    "green": self.green,
                    "blue": self.blue,
                },
            )
        )
        await self.emit(DebugStatus(self, f"{y} {x} {self.chosen}"))


class ColorWidget(Widget):
    value: Reactive = Reactive(255)

    def __init__(self, color: str, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)
        self.color = color
        self.value = 0
        self.fill = 0

    def render(self) -> RenderableType:
        max_h = self._size.height
        max_w = self._size.width - 4
        a = ("█" * max_w + "\n") * self.fill
        b = " \n" * (max_h - self.fill)
        c = a + b
        return Panel(
            c,
            style=self.color,
        )

    def update(self, value):
        self.value = value
        self.fill = value * self._size.height // 255

    async def on_click(self, event: events.Click) -> None:
        max_h = self._size.height - 2
        self.fill = minmax(event.y, 0, max_h)
        self.value = 255 * self.fill // max_h
        await self.emit(ColorStatus(self, {self.color: self.value}))


class StatusWidget(Widget):
    alive: Reactive = Reactive(True)
    points: Reactive = Reactive(0)
    pos: Reactive = Reactive((0, 0))
    debug: Reactive = Reactive("")

    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)

    def render(self) -> RenderableType:
        w = (
            f"Alive: [b red]{self.alive}[/b red] :red_heart:"
            f"\n\n"
            f"Points: [b red]{self.points}[/b red] :glowing_star:"
            f"\n\n"
            f"Pos: [b red]{self.pos}[/b red]"
            f"\n\n"
            f"[b blue]{self.debug}[/b blue]"
        )
        return Panel(
            Align.center(
                w,
                vertical="middle",
            ),
            title="Status",
            style="yellow",
        )


class ToolsWidget(GridView):
    def __init__(
        self,
        *args,
        **kwargs,
    ) -> None:
        super().__init__(*args, **kwargs)
        ...

    def on_mount(self) -> None:
        self.grid.set_gap(1, 1)
        self.grid.set_gutter(0)
        self.grid.set_align("center", "center")

        self.grid.add_column("cola", min_size=7, max_size=7)
        self.grid.add_column("colb", min_size=7, max_size=7)
        self.grid.add_column("colc", min_size=7, max_size=7)
        self.grid.add_column("cold", min_size=6, max_size=6)
        self.grid.add_column("cole", min_size=6, max_size=6)
        self.grid.add_row("row", repeat=2)

        self.grid.add_areas(new="cold-start|cole-end,row1")
        self.grid.add_areas(pick="cold-start|cole-end,row2")
        self.grid.add_areas(paint="cola-start|colc-end,row2")

        self.g8 = MyButton(label="8 \n x\n  8", style="red", value="8")
        self.g16 = MyButton(label="16 \n x\n 16", style="green", value="16")
        self.g32 = MyButton(label="32 \n x\n 32", style="blue", value="32")

        self.new = MyButton(
            label=Align("New\n  Canvas", align="center"), style="white", value="new"
        )
        self.pick_tool = MyButton(label="Pick \n  Color", style="yellow", value="pick")
        self.paint_tool = MyButton(label="\nPaint", style="yellow", value="paint")

        self.grid.place(self.new, self.g8, self.g16, self.g32)

        self.grid.place(new=self.new)
        self.grid.place(pick=self.pick_tool)
        self.grid.place(paint=self.paint_tool)



from textual.widget import Widget
from textual.message import Message
from rich.segment import Segment
from rich.style import Style
from rich.color import Color
from rich.console import Console
from rich.console import ConsoleOptions
from rich.console import (
    Console,
    ConsoleOptions,
    RenderResult,
    RenderableType,
)
from textual.message import Message
from textual import events
from textual.reactive import Reactive
from enum import Enum
from typing import List
from abc import ABC

from pixelart_tui.my_widgets import ColorStatus


class Grid(Enum):
    g8x8 = 4
    g16x16 = 2
    g32x32 = 1


class MouseStatus(Message):
    def __init__(self, sender: Widget, pos: tuple):
        super().__init__(sender)
        self.pos = pos


class ColorUnderMouse(Message):
    def __init__(self, sender: Widget, change: bool, color: Color):
        super().__init__(sender)
        self.change: bool = change
        self.color: Color = color


class Tool(ABC):
    current_color: Color = Color.from_rgb(0, 0, 0)
    mouse_pos: tuple = (0, 0)

    def __init__(self, canvas, color) -> None:
        self.canvas = canvas
        self.current_color = color
        self.under_mouse_color: Color = canvas.matrix[0][0]

    async def on_mouse_down(self, event: events.MouseDown):
        ...

    async def on_mouse_up(self, event: events.MouseUp):
        ...

    async def on_mouse_move(self, event: events.MouseMove):
        ...

    async def on_click(self, event: events.Click):
        ...

    async def on_leave(self, event):
        ...

    def set_color(self, color: Color):
        self.current_color = color

    async def show_on_grid(self, xx, yy):
        y, x = self.mouse_pos
        if y != -1:
            self.canvas.matrix[y][x] = self.under_mouse_color
        self.mouse_pos = (
            yy // self.canvas.grid.value,
            xx // (2 * self.canvas.grid.value),
        )
        y, x = self.mouse_pos
        self.under_mouse_color = self.canvas.matrix[y][x]
        self.canvas.matrix[y][x] = self.current_color
        await self.canvas.emit(MouseStatus(self.canvas, self.mouse_pos))
        self.canvas.update()


class ToolPaint(Tool):
    is_painting: bool = False

    def __init__(self, canvas, color) -> None:
        super().__init__(canvas, color)

    async def on_mouse_down(self, event: events.MouseDown):
        self.is_painting = True

    async def on_mouse_up(self, event: events.MouseUp):
        self.is_painting = False

    async def on_mouse_move(self, event: events.MouseMove):
        if self.is_painting:
            self.under_mouse_color = self.current_color

        await self.show_on_grid(event.x, event.y)

    async def on_click(self, event: events.Click):
        self.under_mouse_color = self.current_color

    async def on_leave(self, event):
        y, x = self.mouse_pos
        self.canvas.matrix[y][x] = self.under_mouse_color
        self.mouse_pos = (-1, -1)
        self.canvas.update()


class ToolPick(Tool):
    async def on_mouse_down(self, event: events.MouseDown):
        ...

    async def on_mouse_up(self, event: events.MouseUp):
        ...

    async def on_mouse_move(self, event: events.MouseMove):
        await self.show_on_grid(event.x, event.y)

        await self.canvas.emit(
            ColorUnderMouse(self.canvas, False, self.under_mouse_color)
        )

    async def on_click(self, event: events.Click):
        await self.canvas.emit(
            ColorUnderMouse(self.canvas, True, self.under_mouse_color)
        )

    async def on_leave(self, event):
        y, x = self.mouse_pos
        self.canvas.matrix[y][x] = self.under_mouse_color
        self.mouse_pos = (-1, -1)
        await self.canvas.emit(ColorStatus(self, {}))
        self.canvas.update()


class Tools(Enum):
    PAINT = ToolPaint
    PICK = ToolPick


class Canvas(Widget):
    """The digital display of the calculator."""

    matrix: Reactive[List[List[Color]]] = Reactive([[]])
    mouse_pos: tuple = (0, 0)
    current_color: Color = Color.from_rgb(0, 0, 0)
    is_painting: bool = False

    def __init__(
        self, w: int = 60, h: int = 60, grid: Grid = Grid.g8x8, *args, **kwargs
    ) -> None:
        super().__init__(*args, **kwargs)
        self.grid = grid
        self.w = w // 2
        self.h = h // 2
        self.matrix = [
            [Color.from_rgb(0, 0, 32 + 32 * ((-1) ** (y + x))) for y in range(h)]
            for x in range(w)
        ]
        self.under_mouse_color = self.matrix[0][0]
        self.tool: Tool = ToolPaint(self, self.current_color)

    def set_tool(self, tool: Tools):
        self.tool = tool.value(self, self.current_color)

    async def on_mouse_move(self, event: events.MouseMove) -> None:
        await self.tool.on_mouse_move(event)

    async def on_click(self, event: events.Click) -> None:
        await self.tool.on_click(event)

    async def on_mouse_down(self, event: events.MouseDown) -> None:
        await self.tool.on_mouse_down(event)

    async def on_mouse_up(self, event: events.MouseUp) -> None:
        await self.tool.on_mouse_up(event)

    async def on_leave(self, event):
        await self.tool.on_leave(event)

    def set_color(self, color: Color):
        self.current_color = color
        self.tool.set_color(color)

    def update(self):
        self.refresh()

    def render(self) -> RenderableType:
        return self

    def set_matrix(self):
        w, h = self.w * 2, self.h * 2
        self.matrix = [
            [Color.from_rgb(0, 0, 32 + 32 * ((-1) ** (y + x))) for y in range(h)]
            for x in range(w)
        ]

    def __rich_console__(
        self, console: Console, options: ConsoleOptions
    ) -> RenderResult:

        for y in range(0, self.h):
            y = y // self.grid.value
            for x in range(0, self.w):
                bgcolor = self.matrix[y][x]
                color = self.matrix[y][x]
                yield Segment(
                    "▄▄" * self.grid.value, Style(bgcolor=bgcolor, color=color)
                )
            yield Segment.line()



from textual.events import Message
from textual.widget import Widget


class DebugStatus(Message):
    def __init__(self, sender: Widget, mes: str):
        super().__init__(sender)
        self.mes = mes


class ColorStatus(Message):
    def __init__(self, sender: Widget, color: dict):
        super().__init__(sender)
        self.color = color


class LoadSaveStatus(Message):
    def __init__(self, sender: Widget, loadsave: str):
        super().__init__(sender)
        self.loadsave = loadsave



from setuptools import setup

from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name="pixelart-tui",
    version="0.2",
    description="Terminal based app for Pixel Art that supports mouse!",
    long_description=long_description,
    long_description_content_type='text/markdown',
    url="http://github.com/Cvaniak/PixelArtTUI",
    author="Cvaniak",
    author_email="igna.cwaniak@gmail.com",
    packages=["pixelart_tui"],
    install_requires=["rich==11.2.0", "textual==0.1.15", "textual-inputs==0.2.5"],
    entry_points={"console_scripts": ["pixelart-tui=pixelart_tui.command_line:run"]},
    zip_safe=False,
)



import os
from rich.color import Color


def save_pxl(matrix, file_name):
    with open(file_name, "w") as file:
        for row in matrix:
            for rgb in row:
                t = str(tuple(rgb.get_truecolor())).replace(" ", "")
                t = t[1:-1]
                # r, g, b = t[0], t[1], t[2]
                file.write(f"{t} ")
            file.write(f"\n")


def load_pxl(matrix, file_name):
    if not os.path.exists(file_name):
        return
    with open(file_name, "r") as file:
        r = file.read().splitlines()
        for iidx, i in enumerate(r):
            j = i.split()
            for jidx, k in enumerate(j):
                tri = eval(k)
                r, g, b = tri
                matrix[iidx][jidx] = Color.from_rgb(red=r, green=g, blue=b)


def save_pal(matrix, file_name):
    with open(file_name, "w") as file:
        for rgb in matrix:
            t = str(tuple(rgb.get_truecolor())).replace(" ", "")
            t = t[1:-1]
            file.write(f"{t} ")
        file.write(f"\n")


def load_pal(matrix, file_name):
    if not os.path.exists(file_name):
        return
    with open(file_name, "r") as file:
        r = file.read().splitlines()
        for i in r:
            j = i.split()
            for jidx, k in enumerate(j):
                tri = eval(k)
                r, g, b = tri
                matrix[jidx] = Color.from_rgb(red=r, green=g, blue=b)
